@@ -4,8 +4,9 @@ use Mouse::Util qw(:meta); # enables strict and warnings
 use Carp ();
 
 use Mouse::Meta::TypeConstraint;
-use Mouse::Meta::Method::Accessor;
 
+#use Mouse::Meta::Method::Accessor;
+use Mouse::Meta::Method::Delegation;
 
 sub _process_options{
     my($class, $name, $args) = @_;
@@ -156,48 +157,8 @@ sub new {
     return $self;
 }
 
-# readers
-
-sub name                 { $_[0]->{name}                   }
-sub associated_class     { $_[0]->{associated_class}       }
-
-sub accessor             { $_[0]->{accessor}               }
-sub reader               { $_[0]->{reader}                 }
-sub writer               { $_[0]->{writer}                 }
-sub predicate            { $_[0]->{predicate}              }
-sub clearer              { $_[0]->{clearer}                }
-sub handles              { $_[0]->{handles}                }
-
-sub _is_metadata         { $_[0]->{is}                     }
-sub is_required          { $_[0]->{required}               }
-sub default              { $_[0]->{default}                }
-sub is_lazy              { $_[0]->{lazy}                   }
-sub is_lazy_build        { $_[0]->{lazy_build}             }
-sub is_weak_ref          { $_[0]->{weak_ref}               }
-sub init_arg             { $_[0]->{init_arg}               }
-sub type_constraint      { $_[0]->{type_constraint}        }
-
-sub trigger              { $_[0]->{trigger}                }
-sub builder              { $_[0]->{builder}                }
-sub should_auto_deref    { $_[0]->{auto_deref}             }
-sub should_coerce        { $_[0]->{coerce}                 }
-
-# predicates
-
-sub has_accessor         { exists $_[0]->{accessor}        }
-sub has_reader           { exists $_[0]->{reader}          }
-sub has_writer           { exists $_[0]->{writer}          }
-sub has_predicate        { exists $_[0]->{predicate}       }
-sub has_clearer          { exists $_[0]->{clearer}         }
-sub has_handles          { exists $_[0]->{handles}         }
-
-sub has_default          { exists $_[0]->{default}         }
-sub has_type_constraint  { exists $_[0]->{type_constraint} }
-sub has_trigger          { exists $_[0]->{trigger}         }
-sub has_builder          { exists $_[0]->{builder}         }
-
-sub has_read_method      { exists $_[0]->{reader} || exists $_[0]->{accessor} }
-sub has_write_method     { exists $_[0]->{writer} || exists $_[0]->{accessor} }
+sub has_read_method      { $_[0]->has_reader || $_[0]->has_accessor }
+sub has_write_method     { $_[0]->has_writer || $_[0]->has_accessor }
 
 sub _create_args { # DEPRECATED
     $_[0]->{_create_args} = $_[1] if @_ > 1;
@@ -243,7 +204,7 @@ sub canonicalize_args{ # DEPRECATED
 
     Carp::cluck("$self->canonicalize_args has been deprecated."
         . "Use \$self->_process_options instead.")
-            if _MOUSE_VERBOSE;
+            if Mouse::Util::_MOUSE_VERBOSE;
 
     return %args;
 }
@@ -253,7 +214,7 @@ sub create { # DEPRECATED
 
     Carp::cluck("$self->create has been deprecated."
         . "Use \$meta->add_attribute and \$attr->install_accessors instead.")
-            if _MOUSE_VERBOSE;
+            if Mouse::Util::_MOUSE_VERBOSE;
 
     # noop
     return $self;
@@ -305,7 +266,13 @@ sub clone_and_inherit_options{
     my($attribute_class, @traits) = ref($self)->interpolate_class(\%args);
 
     $args{traits} = \@traits if @traits;
-    return $attribute_class->new($self->name, %{$self}, %args);
+    # do not inherit the 'handles' attribute
+    foreach my $name(keys %{$self}){
+        if(!exists $args{$name} && $name ne 'handles'){
+            $args{$name} = $self->{$name};
+        }
+    }
+    return $attribute_class->new($self->name, %args);
 }
 
 sub clone_parent { # DEPRECATED
@@ -316,7 +283,7 @@ sub clone_parent { # DEPRECATED
 
     Carp::cluck("$self->clone_parent has been deprecated."
         . "Use \$meta->add_attribute and \$attr->install_accessors instead.")
-        if _MOUSE_VERBOSE;
+        if Mouse::Util::_MOUSE_VERBOSE;
 
     $self->clone_and_inherited_args($class, $name, %args);
 }
@@ -337,10 +304,10 @@ sub get_parent_args { # DEPRECATED
 
 
 sub get_read_method {
-    $_[0]->{reader} || $_[0]->{accessor}
+    $_[0]->reader || $_[0]->accessor
 }
 sub get_write_method {
-    $_[0]->{writer} || $_[0]->{accessor}
+    $_[0]->writer || $_[0]->accessor
 }
 
 sub get_read_method_ref{
@@ -392,27 +359,28 @@ sub _canonicalize_handles {
 
         my $meta = Mouse::Meta::Class->initialize("$class_or_role"); # "" for stringify
         return map  { $_ => $_ }
-               grep { $_ ne 'meta' && !Mouse::Object->can($_) && $_ =~ $handles }
-                   $meta->isa('Mouse::Meta::Class') ? $meta->get_all_method_names : $meta->get_method_list;
+               grep { !Mouse::Object->can($_) && $_ =~ $handles }
+                   Mouse::Util::is_a_metarole($meta)
+                        ? $meta->get_method_list
+                        : $meta->get_all_method_names;
     }
     else {
         $self->throw_error("Unable to canonicalize the 'handles' option with $handles");
     }
 }
 
-
 sub associate_method{
-    my ($attribute, $method) = @_;
+    my ($attribute, $method_name) = @_;
     $attribute->{associated_methods}++;
     return;
 }
 
-sub accessor_metaclass(){ 'Mouse::Meta::Method::Accessor' }
+sub delegation_metaclass() { 'Mouse::Meta::Method::Delegation' }
 
 sub install_accessors{
     my($attribute) = @_;
 
-    my $metaclass      = $attribute->{associated_class};
+    my $metaclass      = $attribute->associated_class;
     my $accessor_class = $attribute->accessor_metaclass;
 
     foreach my $type(qw(accessor reader writer predicate clearer)){
@@ -420,25 +388,25 @@ sub install_accessors{
             my $generator = '_generate_' . $type;
             my $code      = $accessor_class->$generator($attribute, $metaclass);
             $metaclass->add_method($attribute->{$type} => $code);
-            $attribute->associate_method($code);
+            $attribute->associate_method($attribute->{$type});
         }
     }
 
     # install delegation
     if(exists $attribute->{handles}){
+        my $delegation_class = $attribute->delegation_metaclass;
         my %handles = $attribute->_canonicalize_handles($attribute->{handles});
         my $reader  = $attribute->get_read_method_ref;
 
         while(my($handle_name, $method_to_call) = each %handles){
-            my $code = $accessor_class->_generate_delegation($attribute, $metaclass,
+            my $code = $delegation_class->_generate_delegation($attribute, $metaclass,
                 $reader, $handle_name, $method_to_call);
 
             $metaclass->add_method($handle_name => $code);
-            $attribute->associate_method($code);
+            $attribute->associate_method($handle_name);
         }
     }
 
-
     if($attribute->can('create') != \&create){
         # backword compatibility
         $attribute->create($metaclass, $attribute->name, %{$attribute});
@@ -455,7 +423,6 @@ sub throw_error{
 }
 
 1;
-
 __END__
 
 =head1 NAME
@@ -464,7 +431,7 @@ Mouse::Meta::Attribute - The Mouse attribute metaclass
 
 =head1 VERSION
 
-This document describes Mouse version 0.40
+This document describes Mouse version 0.41
 
 =head1 METHODS
 
@@ -572,12 +539,12 @@ is equivalent to this:
 
 =back
 
-=head2 C<< associate_method(Method) >>
+=head2 C<< associate_method(MethodName) >>
 
 Associates a method with the attribute. Typically, this is called internally
 when an attribute generates its accessors.
 
-Currently the argument I<Method> is ignored in Mouse.
+Currently the argument I<MethodName> is ignored in Mouse.
 
 =head2 C<< verify_against_type_constraint(Item) -> TRUE | ERROR >>
 
