@@ -3,23 +3,28 @@ use Mouse::Util qw/:meta get_linear_isa not_supported/; # enables strict and war
 
 use Scalar::Util qw/blessed weaken/;
 
-use Mouse::Meta::Method::Constructor;
-use Mouse::Meta::Method::Destructor;
 use Mouse::Meta::Module;
 our @ISA = qw(Mouse::Meta::Module);
 
-sub method_metaclass()    { 'Mouse::Meta::Method'    }
-sub attribute_metaclass() { 'Mouse::Meta::Attribute' }
+sub attribute_metaclass;
+sub method_metaclass;
 
-sub constructor_class()   { 'Mouse::Meta::Method::Constructor' }
-sub destructor_class()    { 'Mouse::Meta::Method::Destructor'  }
+sub constructor_class;
+sub destructor_class;
+
+my @MetaClassTypes = qw(
+    attribute_metaclass
+    method_metaclass
+    constructor_class
+    destructor_class
+);
 
 sub _construct_meta {
     my($class, %args) = @_;
 
-    $args{attributes} ||= {};
-    $args{methods}    ||= {};
-    $args{roles}      ||= [];
+    $args{attributes} = {};
+    $args{methods}    = {};
+    $args{roles}      = [];
 
     $args{superclasses} = do {
         no strict 'refs';
@@ -38,11 +43,9 @@ sub create_anon_class{
     return $self->create(undef, @_);
 }
 
-sub is_anon_class{
-    return exists $_[0]->{anon_serial_id};
-}
+sub is_anon_class;
 
-sub roles { $_[0]->{roles} }
+sub roles;
 
 sub calculate_all_roles {
     my $self = shift;
@@ -58,9 +61,16 @@ sub superclasses {
         foreach my $super(@_){
             Mouse::Util::load_class($super);
             my $meta = Mouse::Util::get_metaclass_by_name($super);
-            if($meta && $meta->isa('Mouse::Meta::Role')){
+
+            next if not defined $meta;
+
+            if(Mouse::Util::is_a_metarole($meta)){
                 $self->throw_error("You cannot inherit from a Mouse Role ($super)");
             }
+
+            next if $self->isa(ref $meta); # _superclass_meta_is_compatible
+
+            $self->_reconcile_with_superclass_meta($meta);
         }
         @{ $self->{superclasses} } = @_;
     }
@@ -68,6 +78,40 @@ sub superclasses {
     return @{ $self->{superclasses} };
 }
 
+sub _reconcile_with_superclass_meta {
+    my($self, $super_meta) = @_;
+
+    my @incompatibles;
+
+    foreach my $metaclass_type(@MetaClassTypes){
+        my $super_c = $super_meta->$metaclass_type();
+        my $self_c  = $self->$metaclass_type();
+
+        if(!$super_c->isa($self_c)){
+            push @incompatibles, ($metaclass_type => $super_c);
+        }
+    }
+
+    my @roles;
+
+    foreach my $role($self->meta->calculate_all_roles){
+        if(!$super_meta->meta->does_role($role->name)){
+            push @roles, $role->name;
+        }
+    }
+
+    #print "reconcile($self vs. $super_meta; @roles; @incompatibles)\n";
+
+    require Mouse::Util::MetaRole;
+    Mouse::Util::MetaRole::apply_metaclass_roles(
+        for_class       => $self,
+        metaclass       => ref $super_meta,
+        metaclass_roles => \@roles,
+        @incompatibles,
+    );
+    return;
+}
+
 sub find_method_by_name{
     my($self, $method_name) = @_;
     defined($method_name)
@@ -144,7 +188,7 @@ sub add_attribute {
     $self->{attributes}{$attr->name} = $attr;
     $attr->install_accessors();
 
-    if(_MOUSE_VERBOSE && !$attr->{associated_methods} && ($attr->{is} || '') ne 'bare'){
+    if(Mouse::Util::_MOUSE_VERBOSE && !$attr->{associated_methods} && ($attr->{is} || '') ne 'bare'){
         Carp::cluck(qq{Attribute (}.$attr->name.qq{) of class }.$self->name.qq{ has no associated methods (did you mean to provide an "is" argument?)});
     }
     return $attr;
@@ -152,92 +196,13 @@ sub add_attribute {
 
 sub compute_all_applicable_attributes {
     Carp::cluck('compute_all_applicable_attributes() has been deprecated')
-        if _MOUSE_VERBOSE;
+        if Mouse::Util::_MOUSE_VERBOSE;
     return shift->get_all_attributes(@_)
 }
 
-sub get_all_attributes {
-    my $self = shift;
-    my (@attr, %seen);
+sub linearized_isa;
 
-    for my $class ($self->linearized_isa) {
-        my $meta = Mouse::Util::get_metaclass_by_name($class)
-            or next;
-
-        for my $name ($meta->get_attribute_list) {
-            next if $seen{$name}++;
-            push @attr, $meta->get_attribute($name);
-        }
-    }
-
-    return @attr;
-}
-
-sub linearized_isa { @{ get_linear_isa($_[0]->name) } }
-
-sub new_object {
-    my $self = shift;
-    my %args = (@_ == 1 ? %{$_[0]} : @_);
-
-    my $object = bless {}, $self->name;
-
-    $self->_initialize_object($object, \%args);
-    return $object;
-}
-
-sub _initialize_object{
-    my($self, $object, $args) = @_;
-
-    my @triggers_queue;
-
-    foreach my $attribute ($self->get_all_attributes) {
-        my $from = $attribute->init_arg;
-        my $key  = $attribute->name;
-
-        if (defined($from) && exists($args->{$from})) {
-            $object->{$key} = $attribute->_coerce_and_verify($args->{$from}, $object);
-
-            weaken($object->{$key})
-                if ref($object->{$key}) && $attribute->is_weak_ref;
-
-            if ($attribute->has_trigger) {
-                push @triggers_queue, [ $attribute->trigger, $object->{$key} ];
-            }
-        }
-        else {
-            if ($attribute->has_default || $attribute->has_builder) {
-                unless ($attribute->is_lazy) {
-                    my $default = $attribute->default;
-                    my $builder = $attribute->builder;
-                    my $value =   $builder                ? $object->$builder()
-                                : ref($default) eq 'CODE' ? $object->$default()
-                                :                           $default;
-
-                    $object->{$key} = $attribute->_coerce_and_verify($value, $object);
-
-                    weaken($object->{$key})
-                        if ref($object->{$key}) && $attribute->is_weak_ref;
-                }
-            }
-            else {
-                if ($attribute->is_required) {
-                    $self->throw_error("Attribute (".$attribute->name.") is required");
-                }
-            }
-        }
-    }
-
-    foreach my $trigger_and_value(@triggers_queue){
-        my($trigger, $value) = @{$trigger_and_value};
-        $trigger->($object, $value);
-    }
-
-    if($self->is_anon_class){
-        $object->{__METACLASS__} = $self;
-    }
-
-    return $object;
-}
+sub new_object;
 
 sub clone_object {
     my $class  = shift;
@@ -257,29 +222,41 @@ sub clone_instance {
     my ($class, $instance, %params) = @_;
 
     Carp::cluck('clone_instance has been deprecated. Use clone_object instead')
-        if _MOUSE_VERBOSE;
+        if Mouse::Util::_MOUSE_VERBOSE;
     return $class->clone_object($instance, %params);
 }
 
-sub make_immutable {
-    my $self = shift;
-    my %args = (
+
+sub immutable_options {
+    my ( $self, @args ) = @_;
+
+    return (
         inline_constructor => 1,
         inline_destructor  => 1,
         constructor_name   => 'new',
-        @_,
+        @args,
     );
+}
+
+
+sub make_immutable {
+    my $self = shift;
+    my %args = $self->immutable_options(@_);
 
     $self->{is_immutable}++;
 
     if ($args{inline_constructor}) {
+        my $c = $self->constructor_class;
+        Mouse::Util::load_class($c);
         $self->add_method($args{constructor_name} =>
-            $self->constructor_class->_generate_constructor($self, \%args));
+            $c->_generate_constructor($self, \%args));
     }
 
     if ($args{inline_destructor}) {
+        my $c = $self->destructor_class;
+        Mouse::Util::load_class($c);
         $self->add_method(DESTROY =>
-            $self->destructor_class->_generate_destructor($self, \%args));
+            $c->_generate_destructor($self, \%args));
     }
 
     # Moose's make_immutable returns true allowing calling code to skip setting an explicit true value
@@ -481,7 +458,7 @@ Mouse::Meta::Class - The Mouse class metaclass
 
 =head1 VERSION
 
-This document describes Mouse version 0.40
+This document describes Mouse version 0.41
 
 =head1 METHODS
 
@@ -566,6 +543,8 @@ Throws an error with the given message.
 
 =head1 SEE ALSO
 
+L<Mouse::Meta::Module>
+
 L<Moose::Meta::Class>
 
 L<Class::MOP::Class>
