@@ -1,9 +1,13 @@
 package Mouse::Meta::Method::Accessor;
-use Mouse::Util; # enables strict and warnings
-use Scalar::Util qw(blessed);
+use Mouse::Util qw(:meta); # enables strict and warnings
 
-sub _generate_accessor{
-    my (undef, $attribute, $class, $type) = @_;
+sub _inline_slot{
+    my(undef, $self_var, $attr_name) = @_;
+    return sprintf '%s->{q{%s}}', $self_var, $attr_name;
+}
+
+sub _generate_accessor_any{
+    my($method_class, $type, $attribute, $class) = @_;
 
     my $name          = $attribute->name;
     my $default       = $attribute->default;
@@ -17,16 +21,13 @@ sub _generate_accessor{
     my $compiled_type_constraint = defined($constraint) ? $constraint->_compiled_type_constraint : undef;
 
     my $self  = '$_[0]';
-    my $key   = "q{$name}";
-    my $slot  = "$self\->{$key}";
+    my $slot  = $method_class->_inline_slot($self, $name);;
 
-    $type ||= 'accessor';
-
-    my $accessor = sprintf(qq{#line 1 "%s for %s (%s)"\n}, $type, $name, __FILE__)
+    my $accessor = sprintf(qq{package %s;\n#line 1 "%s for %s (%s)"\n}, $class->name, $type, $name, __FILE__)
                  . "sub {\n";
 
-    if ($type eq 'accessor' || $type eq 'writer') {
-        if($type eq 'accessor'){
+    if ($type eq 'rw' || $type eq 'wo') {
+        if($type eq 'rw'){
             $accessor .= 
                 'if (scalar(@_) >= 2) {' . "\n";
         }
@@ -67,7 +68,7 @@ sub _generate_accessor{
 
         $accessor .= "}\n";
     }
-    elsif($type eq 'reader') {
+    elsif($type eq 'ro') {
         $accessor .= 'Carp::confess("Cannot assign a value to a read-only accessor") if scalar(@_) >= 2;' . "\n";
     }
     else{
@@ -93,8 +94,6 @@ sub _generate_accessor{
         }
         elsif(defined $constraint){
             $accessor .= "my \$tmp = $value;\n";
-            #XXX: The following 'defined and' check is for backward compatibility
-            $accessor .= "defined(\$tmp) and ";
 
             $accessor .= "\$compiled_type_constraint->(\$tmp)";
             $accessor .= " || \$attribute->verify_type_constraint_error(\$name, \$tmp, \$constraint);\n";
@@ -103,6 +102,9 @@ sub _generate_accessor{
         else{
             $accessor .= "$slot = $value;\n";
         }
+        if ($is_weak) {
+            $accessor .= "Scalar::Util::weaken($slot) if ref $slot;\n";
+        }
         $accessor .= "}\n";
     }
 
@@ -120,7 +122,7 @@ sub _generate_accessor{
 
     $accessor .= "return $slot;\n}\n";
 
-    #print "# class ", $class->name, "\n", $accessor, "\n";
+    #print $accessor, "\n";
     my $code;
     my $e = do{
         local $@;
@@ -132,16 +134,20 @@ sub _generate_accessor{
     return $code;
 }
 
-sub _generate_reader{
+sub _generate_accessor{
     my $class = shift;
-    return $class->_generate_accessor(@_, 'reader');
+    return $class->_generate_accessor_any(rw => @_);
 }
 
-sub _generate_writer{
+sub _generate_reader {
     my $class = shift;
-    return $class->_generate_accessor(@_, 'writer');
+    return $class->_generate_accessor_any(ro => @_);
 }
 
+sub _generate_writer {
+    my $class = shift;
+    return $class->_generate_accessor_any(wo => @_);
+}
 
 sub _generate_predicate {
     my (undef, $attribute, $class) = @_;
@@ -156,35 +162,11 @@ sub _generate_clearer {
     my (undef, $attribute, $class) = @_;
 
     my $slot = $attribute->name;
-
-   return sub{
+    return sub{
         delete $_[0]->{$slot};
     };
 }
 
-sub _generate_delegation{
-    my (undef, $attribute, $class, $reader, $handle_name, $method_to_call) = @_;
-
-    return sub {
-        my $instance = shift;
-        my $proxy    = $instance->$reader();
-
-        my $error = !defined($proxy)                ? ' is not defined'
-                  : ref($proxy) && !blessed($proxy) ? qq{ is not an object (got '$proxy')}
-                                                    : undef;
-        if ($error) {
-            $instance->meta->throw_error(
-                "Cannot delegate $handle_name to $method_to_call because "
-                    . "the value of "
-                    . $attribute->name
-                    . $error
-             );
-        }
-        $proxy->$method_to_call(@_);
-    };
-}
-
-
 1;
 __END__
 
@@ -194,7 +176,7 @@ Mouse::Meta::Method::Accessor - A Mouse method generator for accessors
 
 =head1 VERSION
 
-This document describes Mouse version 0.40
+This document describes Mouse version 0.41
 
 =head1 SEE ALSO
 
