@@ -1,5 +1,10 @@
 package Mouse::Meta::Method::Constructor;
-use Mouse::Util; # enables strict and warnings
+use Mouse::Util qw(:meta); # enables strict and warnings
+
+sub _inline_slot{
+    my(undef, $self_var, $attr_name) = @_;
+    return sprintf '%s->{q{%s}}', $self_var, $attr_name;
+}
 
 sub _generate_constructor {
     my ($class, $metaclass, $args) = @_;
@@ -42,7 +47,7 @@ sub _generate_constructor {
 }
 
 sub _generate_processattrs {
-    my ($class, $metaclass, $attrs) = @_;
+    my ($method_class, $metaclass, $attrs) = @_;
     my @res;
 
     my $has_triggers;
@@ -55,9 +60,10 @@ sub _generate_processattrs {
 
         my $init_arg        = $attr->init_arg;
         my $type_constraint = $attr->type_constraint;
+        my $is_weak_ref     = $attr->is_weak_ref;
         my $need_coercion;
 
-        my $instance_slot  = "\$instance->{q{$key}}";
+        my $instance_slot  = $method_class->_inline_slot('$instance', $key);
         my $attr_var       = "\$attrs[$index]";
         my $constraint_var;
 
@@ -73,7 +79,7 @@ sub _generate_processattrs {
             $post_process .= "\$checks[$index]->($instance_slot)";
             $post_process .= "  or $attr_var->verify_type_constraint_error(q{$key}, $instance_slot, $constraint_var);\n";
         }
-        if($attr->is_weak_ref){
+        if($is_weak_ref){
             $post_process .= "Scalar::Util::weaken($instance_slot) if ref $instance_slot;\n";
         }
 
@@ -121,6 +127,9 @@ sub _generate_processattrs {
                 }
 
                 $code .= "$instance_slot = $value;\n";
+                if($is_weak_ref){
+                    $code .= "Scalar::Util::weaken($instance_slot);\n";
+                }
             }
         }
         elsif ($attr->is_required) {
@@ -172,10 +181,7 @@ sub _generate_BUILDALL {
 
     my @code;
     for my $class ($metaclass->linearized_isa) {
-        no strict 'refs';
-        no warnings 'once';
-
-        if (*{ $class . '::BUILD' }{CODE}) {
+        if (Mouse::Util::get_code_ref($class, 'BUILD')) {
             unshift  @code, qq{${class}::BUILD(\$instance, \$args);};
         }
     }
@@ -191,7 +197,7 @@ Mouse::Meta::Method::Constructor - A Mouse method generator for constructors
 
 =head1 VERSION
 
-This document describes Mouse version 0.40
+This document describes Mouse version 0.41
 
 =head1 SEE ALSO
 
