@@ -1,11 +1,20 @@
 package Mouse::Meta::Module;
-use Mouse::Util qw/:meta get_code_package load_class not_supported/; # enables strict and warnings
+use Mouse::Util qw/:meta get_code_package get_code_ref load_class not_supported/; # enables strict and warnings
 
-use Carp ();
-use Scalar::Util qw/blessed weaken/;
+use Carp         ();
+use Scalar::Util ();
 
 my %METAS;
 
+# XXX: work around a warning "useless use of a constant in void context" in 5.6.2
+if(&Mouse::Util::MOUSE_XS){
+    # register meta storage for performance
+    Mouse::Util::__register_metaclass_storage(\%METAS, 0);
+
+    # ensure thread safety
+    *CLONE = sub { Mouse::Util::__register_metaclass_storage(\%METAS, 1) };
+}
+
 sub _metaclass_cache { # DEPRECATED
     my($class, $name) = @_;
     return $METAS{$name};
@@ -21,7 +30,19 @@ sub initialize {
         ||= $class->_construct_meta(package => $package_name, @args);
 }
 
-sub class_of{
+sub reinitialize {
+    my($class, $package_name, @args) = @_;
+
+    $package_name = $package_name->name if ref $package_name;
+
+    ($package_name && !ref($package_name))
+        || $class->throw_error("You must pass a package name and it cannot be blessed");
+
+    delete $METAS{$package_name};
+    return $class->initialize($package_name, @args);
+}
+
+sub _class_of{
     my($class_or_instance) = @_;
     return undef unless defined $class_or_instance;
     return $METAS{ ref($class_or_instance) || $class_or_instance };
@@ -29,18 +50,18 @@ sub class_of{
 
 # Means of accessing all the metaclasses that have
 # been initialized thus far
-#sub get_all_metaclasses         {        %METAS         }
-sub get_all_metaclass_instances { values %METAS         }
-sub get_all_metaclass_names     { keys   %METAS         }
-sub get_metaclass_by_name       { $METAS{$_[0]}         }
-#sub store_metaclass_by_name     { $METAS{$_[0]} = $_[1] }
-#sub weaken_metaclass            { weaken($METAS{$_[0]}) }
-#sub does_metaclass_exist        { defined $METAS{$_[0]} }
-#sub remove_metaclass_by_name    { delete $METAS{$_[0]}  }
+#sub _get_all_metaclasses         {        %METAS         }
+sub _get_all_metaclass_instances { values %METAS         }
+sub _get_all_metaclass_names     { keys   %METAS         }
+sub _get_metaclass_by_name       { $METAS{$_[0]}         }
+#sub _store_metaclass_by_name     { $METAS{$_[0]} = $_[1] }
+#sub _weaken_metaclass            { weaken($METAS{$_[0]}) }
+#sub _does_metaclass_exist        { defined $METAS{$_[0]} }
+#sub _remove_metaclass_by_name    { delete $METAS{$_[0]}  }
 
+sub name;
 
-
-sub name { $_[0]->{package} }
+sub namespace;
 
 # The followings are Class::MOP specific methods
 
@@ -64,96 +85,59 @@ sub get_attribute_map { # DEPRECATED
 
 sub has_attribute     { exists $_[0]->{attributes}->{$_[1]} }
 sub get_attribute     {        $_[0]->{attributes}->{$_[1]} }
-sub get_attribute_list{ keys %{$_[0]->{attributes}}         }
 sub remove_attribute  { delete $_[0]->{attributes}->{$_[1]} }
 
-sub namespace{
-    my $name = $_[0]->{package};
-    no strict 'refs';
-    return \%{ $name . '::' };
-}
-
-sub add_method {
-    my($self, $name, $code) = @_;
-
-    if(!defined $name){
-        $self->throw_error('You must pass a defined name');
-    }
-    if(!defined $code){
-        $self->throw_error('You must pass a defined code');
-    }
-
-    if(ref($code) ne 'CODE'){
-        $code = \&{$code}; # coerce
-    }
-
-    $self->{methods}->{$name} = $code; # Moose stores meta object here.
-
-    my $pkg = $self->name;
-    no strict 'refs';
-    no warnings 'redefine', 'once';
-    *{ $pkg . '::' . $name } = $code;
-}
+sub get_attribute_list{ keys   %{$_[0]->{attributes}} }
 
 # XXX: for backward compatibility
 my %foreign = map{ $_ => undef } qw(
     Mouse Mouse::Role Mouse::Util Mouse::Util::TypeConstraints
-    Carp Scalar::Util
+    Carp Scalar::Util List::Util
 );
 sub _code_is_mine{
-    my($self, $code) = @_;
+#    my($self, $code) = @_;
 
-    my $package = get_code_package($code);
-
-    return !exists $foreign{$package};
+    return !exists $foreign{ get_code_package($_[1]) };
 }
 
+sub add_method;
+
 sub has_method {
     my($self, $method_name) = @_;
 
     defined($method_name)
         or $self->throw_error('You must define a method name');
 
-    return 1 if $self->{methods}{$method_name};
-
-    my $code = do{
-        no strict 'refs';
-        no warnings 'once';
-        *{ $self->{package} . '::' . $method_name }{CODE};
+    return defined($self->{methods}{$method_name}) || do{
+        my $code = get_code_ref($self->{package}, $method_name);
+        $code && $self->_code_is_mine($code);
     };
-
-    return $code && $self->_code_is_mine($code);
 }
 
-sub get_method_body{
+sub get_method_body {
     my($self, $method_name) = @_;
 
     defined($method_name)
         or $self->throw_error('You must define a method name');
 
     return $self->{methods}{$method_name} ||= do{
-        my $code = do{
-            no strict 'refs';
-            no warnings 'once';
-            *{$self->{package} . '::' . $method_name}{CODE};
-        };
-
-        ($code && $self->_code_is_mine($code)) ? $code : undef;
+        my $code = get_code_ref($self->{package}, $method_name);
+        $code && $self->_code_is_mine($code) ? $code : undef;
     };
 }
 
 sub get_method{
     my($self, $method_name) = @_;
 
-    if($self->has_method($method_name)){
+    if(my $code = $self->get_method_body($method_name)){
         my $method_metaclass = $self->method_metaclass;
         load_class($method_metaclass);
 
-        my $package = $self->name;
-        return $method_metaclass->new(
-            body    => $package->can($method_name),
-            name    => $method_name,
-            package => $package,
+        return $method_metaclass->wrap(
+            body                 => $code,
+            name                 => $method_name,
+            package              => $self->name,
+            associated_metaclass => $self,
         );
     }
 
@@ -179,7 +163,7 @@ sub get_method_list {
 
         my $superclasses;
         if(exists $options{superclasses}){
-            if($self->isa('Mouse::Meta::Role')){
+            if(Mouse::Util::is_a_metarole($self)){
                 delete $options{superclasses};
             }
             else{
@@ -232,7 +216,7 @@ sub get_method_list {
 
         my $meta = $self->initialize( $package_name, %options);
 
-        weaken $METAS{$package_name}
+        Scalar::Util::weaken $METAS{$package_name}
             if $mortal;
 
         $meta->add_method(meta => sub{
@@ -280,6 +264,8 @@ sub get_method_list {
     sub DESTROY{
         my($self) = @_;
 
+        return if $Mouse::Util::in_global_destruction;
+
         my $serial_id = $self->{anon_serial_id};
 
         return if !$serial_id;
@@ -317,7 +303,6 @@ sub throw_error{
 }
 
 1;
-
 __END__
 
 =head1 NAME
@@ -326,7 +311,7 @@ Mouse::Meta::Module - The base class for Mouse::Meta::Class and Mouse::Meta::Rol
 
 =head1 VERSION
 
-This document describes Mouse version 0.40
+This document describes Mouse version 0.41
 
 =head1 SEE ALSO
 
