@@ -4,6 +4,16 @@ use Mouse::Util; # enables strict and warnings
 use Mouse::Meta::Method;
 our @ISA = qw(Mouse::Meta::Method);
 
+sub _new{
+    my($class, %args) = @_;
+    my $self = bless \%args, $class;
+
+    if($class ne __PACKAGE__){
+        $self->meta->_initialize_object($self, \%args);
+    }
+    return $self;
+}
+
 1;
 __END__
 
@@ -13,7 +23,7 @@ Mouse::Meta::Role::Method - A Mouse Method metaclass for Roles
 
 =head1 VERSION
 
-This document describes Mouse version 0.40
+This document describes Mouse version 0.41
 
 =head1 SEE ALSO
 
