@@ -2,13 +2,20 @@ package Mouse::Meta::TypeConstraint;
 use Mouse::Util qw(:meta); # enables strict and warnings
 
 use overload
-    '""'     => sub { shift->{name} },   # stringify to tc name
+    'bool'   => sub { 1 },             # always true
+
+    '""'     => sub { $_[0]->name },   # stringify to tc name
+
+    '|'      => sub {                  # or-combination
+        require Mouse::Util::TypeConstraints;
+        return Mouse::Util::TypeConstraints::find_or_parse_type_constraint(
+            "$_[0] | $_[1]",
+        );
+    },
+
     fallback => 1;
 
-use Carp qw(confess);
-use Scalar::Util qw(blessed reftype);
-
-my $null_check = sub { 1 };
+use Carp         ();
 
 sub new {
     my($class, %args) = @_;
@@ -19,7 +26,7 @@ sub new {
 
     if($args{_compiled_type_constraint}){
         Carp::cluck("'_compiled_type_constraint' has been deprecated, use 'optimized' instead")
-            if _MOUSE_VERBOSE;
+            if Mouse::Util::_MOUSE_VERBOSE;
 
         $check = $args{_compiled_type_constraint};
     }
@@ -31,13 +38,8 @@ sub new {
 
     $check = $args{constraint};
 
-    if(blessed($check)){
-        Carp::cluck("Constraint for $args{name} must be a CODE reference");
-        $check = $check->{compiled_type_constraint};
-    }
-
     if(defined($check) && ref($check) ne 'CODE'){
-        confess("Constraint for $args{name} is not a CODE reference");
+        Carp::confess("Constraint for $args{name} is not a CODE reference");
     }
 
     $args{package_defined_in} ||= caller;
@@ -86,68 +88,6 @@ sub create_child_type{
    );
 }
 
-sub name    { $_[0]->{name}    }
-sub parent  { $_[0]->{parent}  }
-sub message { $_[0]->{message} }
-
-sub _compiled_type_constraint{ $_[0]->{compiled_type_constraint} }
-
-sub has_coercion{ exists $_[0]->{_compiled_type_coercion} }
-
-sub compile_type_constraint{
-    my($self) = @_;
-
-    # add parents first
-    my @checks;
-    for(my $parent = $self->parent; defined $parent; $parent = $parent->parent){
-         if($parent->{hand_optimized_type_constraint}){
-            unshift @checks, $parent->{hand_optimized_type_constraint};
-            last; # a hand optimized constraint must include all the parents
-        }
-        elsif($parent->{constraint}){
-            unshift @checks, $parent->{constraint};
-        }
-    }
-
-    # then add child
-    if($self->{constraint}){
-        push @checks, $self->{constraint};
-    }
-
-    if($self->{type_constraints}){ # Union
-        my @types = map{ $_->_compiled_type_constraint } @{ $self->{type_constraints} };
-        push @checks, sub{
-            foreach my $c(@types){
-                return 1 if $c->($_[0]);
-            }
-            return 0;
-        };
-    }
-
-    if(@checks == 0){
-        $self->{compiled_type_constraint} = $null_check;
-    }
-    elsif(@checks == 1){
-        my $c = $checks[0];
-        $self->{compiled_type_constraint} = sub{
-            my(@args) = @_;
-            local $_ = $args[0];
-            return $c->(@args);
-        };
-    }
-    else{
-        $self->{compiled_type_constraint} =  sub{
-            my(@args) = @_;
-            local $_ = $args[0];
-            foreach my $c(@checks){
-                return undef if !$c->(@args);
-            }
-            return 1;
-        };
-    }
-    return;
-}
-
 sub _add_type_coercions{
     my $self = shift;
 
@@ -159,18 +99,18 @@ sub _add_type_coercions{
         my $action = $_[++$i];
 
         if(exists $has{$from}){
-            confess("A coercion action already exists for '$from'");
+            Carp::confess("A coercion action already exists for '$from'");
         }
 
         my $type = Mouse::Util::TypeConstraints::find_or_parse_type_constraint($from)
-            or confess("Could not find the type constraint ($from) to coerce from");
+            or Carp::confess("Could not find the type constraint ($from) to coerce from");
 
         push @{$coercions}, [ $type => $action ];
     }
 
     # compile
     if(exists $self->{type_constraints}){ # union type
-        confess("Cannot add additional type coercions to Union types");
+        Carp::confess("Cannot add additional type coercions to Union types");
     }
     else{
         $self->{_compiled_type_coercion} = sub {
@@ -195,13 +135,11 @@ sub check {
 
 sub coerce {
     my $self = shift;
-    if(!$self->{_compiled_type_coercion}){
-        confess("Cannot coerce without a type coercion ($self)");
-    }
 
     return $_[0] if $self->_compiled_type_constraint->(@_);
 
-    return $self->{_compiled_type_coercion}->(@_);
+    my $coercion = $self->_compiled_type_coercion;
+    return $coercion ? $coercion->(@_) : $_[0];
 }
 
 sub get_message {
@@ -220,7 +158,7 @@ sub is_a_type_of{
     my($self, $other) = @_;
 
     # ->is_a_type_of('__ANON__') is always false
-    return 0 if !blessed($other) && $other eq '__ANON__';
+    return 0 if !ref($other) && $other eq '__ANON__';
 
     (my $other_name = $other) =~ s/\s+//g;
 
@@ -239,6 +177,29 @@ sub is_a_type_of{
     return 0;
 }
 
+# See also Moose::Meta::TypeConstraint::Parameterizable
+sub parameterize{
+    my($self, $param, $name) = @_;
+
+    if(!ref $param){
+        require Mouse::Util::TypeConstraints;
+        $param = Mouse::Util::TypeConstraints::find_or_create_isa_type_constraint($param);
+    }
+
+    $name ||= sprintf '%s[%s]', $self->name, $param->name;
+
+    my $generator = $self->{constraint_generator}
+        || Carp::confess("The $name constraint cannot be used, because $param doesn't subtype from a parameterizable type");
+
+    return Mouse::Meta::TypeConstraint->new(
+        name        => $name,
+        parent      => $self,
+        parameter   => $param,
+        constraint  => $generator->($param), # must be 'constraint', not 'optimized'
+
+        type        => 'Parameterized',
+    );
+}
 
 1;
 __END__
@@ -249,7 +210,7 @@ Mouse::Meta::TypeConstraint - The Mouse Type Constraint metaclass
 
 =head1 VERSION
 
-This document describes Mouse version 0.40
+This document describes Mouse version 0.41
 
 =head1 DESCRIPTION
 
