@@ -1,48 +1,7 @@
 package Mouse::Object;
-use Mouse::Util qw(does dump); # enables strict and warnings
+use Mouse::Util qw(does dump meta); # enables strict and warnings
 
-sub new {
-    my $class = shift;
-
-    $class->throw_error('Cannot call new() on an instance') if ref $class;
-
-    my $args = $class->BUILDARGS(@_);
-
-    my $instance = Mouse::Meta::Class->initialize($class)->new_object($args);
-    $instance->BUILDALL($args);
-    return $instance;
-}
-
-sub BUILDARGS {
-    my $class = shift;
-
-    if (scalar @_ == 1) {
-        (ref($_[0]) eq 'HASH')
-            || $class->meta->throw_error("Single parameters to new() must be a HASH ref");
-
-        return {%{$_[0]}};
-    }
-    else {
-        return {@_};
-    }
-}
-
-sub DESTROY {
-    my $self = shift;
-
-    local $?;
-
-    my $e = do{
-        local $@;
-        eval{
-            $self->DEMOLISHALL();
-        };
-        $@;
-    };
-
-    no warnings 'misc';
-    die $e if $e; # rethrow
-}
+sub new;
 
 sub BUILDALL {
     my $self = shift;
@@ -51,11 +10,8 @@ sub BUILDALL {
     return unless $self->can('BUILD');
 
     for my $class (reverse $self->meta->linearized_isa) {
-        my $build = do{
-            no strict 'refs';
-            no warnings 'once';
-            *{ $class . '::BUILD' }{CODE};
-        } or next;
+        my $build = Mouse::Util::get_code_ref($class, 'BUILD')
+            || next;
 
         $self->$build(@_);
     }
@@ -74,11 +30,8 @@ sub DEMOLISHALL {
     # that time (at least tests suggest so ;)
 
     foreach my $class (@{ Mouse::Util::get_linear_isa(ref $self) }) {
-        my $demolish = do{
-            no strict 'refs';
-            no warnings 'once';
-            *{ $class . '::DEMOLISH'}{CODE};
-        } or next;
+        my $demolish = Mouse::Util::get_code_ref($class, 'DEMOLISH')
+            || next;
 
         $self->$demolish();
     }
@@ -86,7 +39,6 @@ sub DEMOLISHALL {
 }
 
 1;
-
 __END__
 
 =head1 NAME
@@ -95,7 +47,7 @@ Mouse::Object - The base object for Mouse classes
 
 =head1 VERSION
 
-This document describes Mouse version 0.40
+This document describes Mouse version 0.41
 
 =head1 METHODS
 
