@@ -2,7 +2,8 @@ package Mouse::PurePerl;
 
 require Mouse::Util;
 
-package Mouse::Util;
+package
+    Mouse::Util;
 
 use strict;
 use warnings;
@@ -11,19 +12,6 @@ use warnings FATAL => 'redefine'; # to avoid to load Mouse::PurePerl
 
 use B ();
 
-
-# taken from Class/MOP.pm
-sub is_valid_class_name {
-    my $class = shift;
-
-    return 0 if ref($class);
-    return 0 unless defined($class);
-
-    return 1 if $class =~ /\A \w+ (?: :: \w+ )* \z/xms;
-
-    return 0;
-}
-
 sub is_class_loaded {
     my $class = shift;
 
@@ -101,40 +89,17 @@ sub generate_isa_predicate_for {
     my $predicate = sub{ Scalar::Util::blessed($_[0]) && $_[0]->isa($for_class) };
 
     if(defined $name){
-        Mouse::Util::install_subroutines(scalar caller, $name => $predicate);
+        no strict 'refs';
+        *{ caller() . '::' . $name } = $predicate;
         return;
     }
 
     return $predicate;
 }
 
-sub generate_can_predicate_for {
-    my($methods_ref, $name) = @_;
 
-    my @methods = @{$methods_ref};
-
-    my $predicate = sub{
-        my($instance) = @_;
-        if(Scalar::Util::blessed($instance)){
-            foreach my $method(@methods){
-                if(!$instance->can($method)){
-                    return 0;
-                }
-            }
-            return 1;
-        }
-        return 0;
-    };
-
-    if(defined $name){
-        Mouse::Util::install_subroutines(scalar caller, $name => $predicate);
-        return;
-    }
-
-    return $predicate;
-}
-
-package Mouse::Util::TypeConstraints;
+package
+    Mouse::Util::TypeConstraints;
 
 use Scalar::Util qw(blessed looks_like_number openhandle);
 
@@ -145,21 +110,12 @@ sub Bool       { $_[0] ? $_[0] eq '1' : 1 }
 sub Undef      { !defined($_[0]) }
 sub Defined    {  defined($_[0])  }
 sub Value      {  defined($_[0]) && !ref($_[0]) }
-sub Num        {  looks_like_number($_[0]) }
-sub Int        {
-    my($value) = @_;
-    looks_like_number($value) && $value =~ /\A [+-]? [0-9]+  \z/xms;
-}
-sub Str        {
-    my($value) = @_;
-    return defined($value) && ref(\$value) eq 'SCALAR';
-}
+sub Num        { !ref($_[0]) && looks_like_number($_[0]) }
+sub Int        {  defined($_[0]) && !ref($_[0]) && $_[0] =~ /^-?[0-9]+$/ }
+sub Str        {  defined($_[0]) && !ref($_[0]) }
 
 sub Ref        { ref($_[0]) }
-sub ScalarRef  {
-    my($value) = @_;
-    return ref($value) eq 'SCALAR'
-}
+sub ScalarRef  { ref($_[0]) eq 'SCALAR' }
 sub ArrayRef   { ref($_[0]) eq 'ARRAY'  }
 sub HashRef    { ref($_[0]) eq 'HASH'   }
 sub CodeRef    { ref($_[0]) eq 'CODE'   }
@@ -167,7 +123,7 @@ sub RegexpRef  { ref($_[0]) eq 'Regexp' }
 sub GlobRef    { ref($_[0]) eq 'GLOB'   }
 
 sub FileHandle {
-    return openhandle($_[0])  || (blessed($_[0]) && $_[0]->isa("IO::Handle"))
+    openhandle($_[0])  || (blessed($_[0]) && $_[0]->isa("IO::Handle"))
 }
 
 sub Object     { blessed($_[0]) && blessed($_[0]) ne 'Regexp' }
@@ -207,9 +163,12 @@ sub _parameterize_Maybe_for {
     return sub{
         return !defined($_) || $check->($_);
     };
-}
+};
 
-package Mouse::Meta::Module;
+
+
+package
+    Mouse::Meta::Module;
 
 sub name          { $_[0]->{package} }
 
@@ -238,16 +197,87 @@ sub add_method {
 
     $self->{methods}->{$name} = $code; # Moose stores meta object here.
 
-    Mouse::Util::install_subroutines($self->name,
-        $name => $code,
-    );
+    my $pkg = $self->name;
+    no strict 'refs';
+    no warnings 'redefine', 'once';
+    *{ $pkg . '::' . $name } = $code;
     return;
 }
 
-package Mouse::Meta::Class;
+my %SIGIL_MAP = (
+    '$' => 'SCALAR',
+    '@' => 'ARRAY',
+    '%' => 'HASH',
+    '&' => 'CODE',
+    '*' => 'GLOB',
+);
 
-use Mouse::Meta::Method::Constructor;
-use Mouse::Meta::Method::Destructor;
+sub _deconstruct_variable_name {
+    my($self, $variable) = @_;
+
+    (defined $variable)
+        || $self->throw_error("You must pass a variable name");
+
+    my $sigil = substr($variable, 0, 1, '');
+
+    (defined $sigil)
+        || $self->throw_error("The variable name must include a sigil");
+
+    (exists $SIGIL_MAP{$sigil})
+        || $self->throw_error("I do not recognize that sigil '$sigil'");
+
+    return ($variable, $SIGIL_MAP{$sigil});
+}
+
+sub has_package_symbol {
+    my($self, $variable) = @_;
+
+    my($name, $type) = $self->_deconstruct_variable_name($variable);
+
+    my $namespace = $self->namespace;
+
+    return 0 unless exists $namespace->{$name};
+
+    my $entry_ref = \$namespace->{$name};
+    if ( ref($entry_ref) eq 'GLOB' ) {
+        return defined( *{$entry_ref}{$type} );
+    }
+    else {
+        # a symbol table entry can be -1 (stub), string (stub with prototype),
+        # or reference (constant)
+        return $type eq 'CODE';
+    }
+}
+
+sub get_package_symbol {
+    my ($self, $variable) = @_;
+
+    my($name, $type) = $self->_deconstruct_variable_name($variable);
+
+    my $namespace = $self->namespace;
+
+    return undef
+        unless exists $namespace->{$name};
+
+    my $entry_ref = \$namespace->{$name};
+
+    if ( ref($entry_ref) eq 'GLOB' ) {
+        return *{$entry_ref}{$type};
+    }
+    else {
+        if ( $type eq 'CODE' ) {
+            no strict 'refs';
+            return \&{ $self->name . '::' . $name };
+        }
+        else {
+            return undef;
+        }
+    }
+}
+
+
+package
+    Mouse::Meta::Class;
 
 sub method_metaclass    { $_[0]->{method_metaclass}    || 'Mouse::Meta::Method'    }
 sub attribute_metaclass { $_[0]->{attribute_metaclass} || 'Mouse::Meta::Attribute' }
@@ -333,11 +363,9 @@ sub _initialize_object{
     return;
 }
 
-sub is_immutable {  $_[0]->{is_immutable} }
 
-sub __strict_constructor{ $_[0]->{strict_constructor} }
-
-package Mouse::Meta::Role;
+package
+    Mouse::Meta::Role;
 
 sub method_metaclass{ $_[0]->{method_metaclass} || 'Mouse::Meta::Role::Method' }
 
@@ -347,39 +375,8 @@ sub is_anon_role{
 
 sub get_roles { $_[0]->{roles} }
 
-sub add_before_method_modifier {
-    my ($self, $method_name, $method) = @_;
-
-    push @{ $self->{before_method_modifiers}{$method_name} ||= [] }, $method;
-    return;
-}
-sub add_around_method_modifier {
-    my ($self, $method_name, $method) = @_;
-
-    push @{ $self->{around_method_modifiers}{$method_name} ||= [] }, $method;
-    return;
-}
-sub add_after_method_modifier {
-    my ($self, $method_name, $method) = @_;
-
-    push @{ $self->{after_method_modifiers}{$method_name} ||= [] }, $method;
-    return;
-}
-
-sub get_before_method_modifiers {
-    my ($self, $method_name) = @_;
-    return @{ $self->{before_method_modifiers}{$method_name} ||= [] }
-}
-sub get_around_method_modifiers {
-    my ($self, $method_name) = @_;
-    return @{ $self->{around_method_modifiers}{$method_name} ||= [] }
-}
-sub get_after_method_modifiers {
-    my ($self, $method_name) = @_;
-    return @{ $self->{after_method_modifiers}{$method_name} ||= [] }
-}
-
-package Mouse::Meta::Attribute;
+package
+    Mouse::Meta::Attribute;
 
 require Mouse::Meta::Method::Accessor;
 
@@ -429,146 +426,18 @@ sub has_builder          { exists $_[0]->{builder}         }
 
 sub has_documentation    { exists $_[0]->{documentation}   }
 
-sub _process_options{
-    my($class, $name, $args) = @_;
-
-    # taken from Class::MOP::Attribute::new
-
-    defined($name)
-        or $class->throw_error('You must provide a name for the attribute');
-
-    if(!exists $args->{init_arg}){
-        $args->{init_arg} = $name;
-    }
-
-    # 'required' requires eigher 'init_arg', 'builder', or 'default'
-    my $can_be_required = defined( $args->{init_arg} );
-
-    if(exists $args->{builder}){
-        # XXX:
-        # Moose refuses a CODE ref builder, but Mouse doesn't for backward compatibility
-        # This feature will be changed in a future. (gfx)
-        $class->throw_error('builder must be a defined scalar value which is a method name')
-            #if ref $args->{builder} || !defined $args->{builder};
-            if !defined $args->{builder};
-
-        $can_be_required++;
-    }
-    elsif(exists $args->{default}){
-        if(ref $args->{default} && ref($args->{default}) ne 'CODE'){
-            $class->throw_error("References are not allowed as default values, you must "
-                              . "wrap the default of '$name' in a CODE reference (ex: sub { [] } and not [])");
-        }
-        $can_be_required++;
-    }
-
-    if( $args->{required} && !$can_be_required ) {
-        $class->throw_error("You cannot have a required attribute ($name) without a default, builder, or an init_arg");
-    }
-
-    # taken from Mouse::Meta::Attribute->new and ->_process_args
-
-    if(exists $args->{is}){
-        my $is = $args->{is};
-
-        if($is eq 'ro'){
-            $args->{reader} ||= $name;
-        }
-        elsif($is eq 'rw'){
-            if(exists $args->{writer}){
-                $args->{reader} ||= $name;
-             }
-             else{
-                $args->{accessor} ||= $name;
-             }
-        }
-        elsif($is eq 'bare'){
-            # do nothing, but don't complain (later) about missing methods
-        }
-        else{
-            $is = 'undef' if !defined $is;
-            $class->throw_error("I do not understand this option (is => $is) on attribute ($name)");
-        }
-    }
-
-    my $tc;
-    if(exists $args->{isa}){
-        $tc = $args->{type_constraint} = Mouse::Util::TypeConstraints::find_or_create_isa_type_constraint($args->{isa});
-    }
-
-    if(exists $args->{does}){
-        if(defined $tc){ # both isa and does supplied
-            my $does_ok = do{
-                local $@;
-                eval{ "$tc"->does($args) };
-            };
-            if(!$does_ok){
-                $class->throw_error("Cannot have both an isa option and a does option because '$tc' does not do '$args->{does}' on attribute ($name)");
-            }
-        }
-        else {
-            $tc = $args->{type_constraint} = Mouse::Util::TypeConstraints::find_or_create_does_type_constraint($args->{does});
-        }
-    }
-
-    if($args->{coerce}){
-        defined($tc)
-            || $class->throw_error("You cannot have coercion without specifying a type constraint on attribute ($name)");
-
-        $args->{weak_ref}
-            && $class->throw_error("You cannot have a weak reference to a coerced value on attribute ($name)");
-    }
-
-    if ($args->{lazy_build}) {
-        exists($args->{default})
-            && $class->throw_error("You can not use lazy_build and default for the same attribute ($name)");
-
-        $args->{lazy}      = 1;
-        $args->{builder} ||= "_build_${name}";
-        if ($name =~ /^_/) {
-            $args->{clearer}   ||= "_clear${name}";
-            $args->{predicate} ||= "_has${name}";
-        }
-        else {
-            $args->{clearer}   ||= "clear_${name}";
-            $args->{predicate} ||= "has_${name}";
-        }
-    }
-
-    if ($args->{auto_deref}) {
-        defined($tc)
-            || $class->throw_error("You cannot auto-dereference without specifying a type constraint on attribute ($name)");
-
-        ( $tc->is_a_type_of('ArrayRef') || $tc->is_a_type_of('HashRef') )
-            || $class->throw_error("You cannot auto-dereference anything other than a ArrayRef or HashRef on attribute ($name)");
-    }
-
-    if (exists $args->{trigger}) {
-        ('CODE' eq ref $args->{trigger})
-            || $class->throw_error("Trigger must be a CODE ref on attribute ($name)");
-    }
-
-    if ($args->{lazy}) {
-        (exists $args->{default} || defined $args->{builder})
-            || $class->throw_error("You cannot have lazy attribute ($name) without specifying a default value for it");
-    }
-
-    return;
-}
-
-
-package Mouse::Meta::TypeConstraint;
+package
+    Mouse::Meta::TypeConstraint;
 
 sub name    { $_[0]->{name}    }
 sub parent  { $_[0]->{parent}  }
 sub message { $_[0]->{message} }
 
-sub type_parameter           { $_[0]->{type_parameter} }
 sub _compiled_type_constraint{ $_[0]->{compiled_type_constraint} }
+
 sub _compiled_type_coercion  { $_[0]->{_compiled_type_coercion}  }
 
-sub __is_parameterized { exists $_[0]->{type_parameter} }
-sub has_coercion {       exists $_[0]->{_compiled_type_coercion} }
+sub has_coercion{ exists $_[0]->{_compiled_type_coercion} }
 
 
 sub compile_type_constraint{
@@ -617,7 +486,9 @@ sub compile_type_constraint{
     return;
 }
 
-package Mouse::Object;
+package
+    Mouse::Object;
+
 
 sub BUILDARGS {
     my $class = shift;
@@ -666,6 +537,7 @@ sub DESTROY {
     my $e = do{
         local $@;
         eval{
+
             # DEMOLISHALL
 
             # We cannot count on being able to retrieve a previously made
@@ -677,7 +549,7 @@ sub DESTROY {
                 my $demolish = Mouse::Util::get_code_ref($class, 'DEMOLISH')
                     || next;
 
-                $self->$demolish($Mouse::Util::in_global_destruction);
+                $self->$demolish();
             }
         };
         $@;
@@ -687,24 +559,6 @@ sub DESTROY {
     die $e if $e; # rethrow
 }
 
-sub BUILDALL {
-    my $self = shift;
-
-    # short circuit
-    return unless $self->can('BUILD');
-
-    for my $class (reverse $self->meta->linearized_isa) {
-        my $build = Mouse::Util::get_code_ref($class, 'BUILD')
-            || next;
-
-        $self->$build(@_);
-    }
-    return;
-}
-
-sub DEMOLISHALL;
-*DEMOLISHALL = \&DESTROY;
-
 1;
 __END__
 
@@ -714,7 +568,7 @@ Mouse::PurePerl - A Mouse guts in pure Perl
 
 =head1 VERSION
 
-This document describes Mouse version 0.50_09
+This document describes Mouse version 0.41
 
 =head1 SEE ALSO
 
