@@ -1,13 +1,13 @@
 package Mouse::Role;
 use Mouse::Exporter; # enables strict and warnings
 
-our $VERSION = '0.40';
+our $VERSION = '0.41';
 
 use Carp         qw(confess);
 use Scalar::Util qw(blessed);
 
 use Mouse::Util  qw(not_supported);
-
+use Mouse::Meta::Role;
 use Mouse ();
 
 Mouse::Exporter->setup_import_methods(
@@ -25,18 +25,34 @@ Mouse::Exporter->setup_import_methods(
     ],
 );
 
-# XXX: for backward compatibility
-our @EXPORT = qw(
-    extends with
-    has
-    before after around
-    override super
-    augment  inner
 
-    requires excludes
+sub extends  {
+    Carp::croak "Roles do not support 'extends'";
+}
 
-    blessed confess
-);
+sub with     {
+    my $meta = Mouse::Meta::Role->initialize(scalar caller);
+    Mouse::Util::apply_all_roles($meta->name, @_);
+    return;
+}
+
+sub has {
+    my $meta = Mouse::Meta::Role->initialize(scalar caller);
+    my $name = shift;
+
+    $meta->throw_error(q{Usage: has 'name' => ( key => value, ... )})
+        if @_ % 2; # odd number of arguments
+
+    if(ref $name){ # has [qw(foo bar)] => (...)
+        for (@{$name}){
+            $meta->add_attribute($_ => @_);
+        }
+    }
+    else{ # has foo => (...)
+        $meta->add_attribute($name => @_);
+    }
+    return;
+}
 
 sub before {
     my $meta = Mouse::Meta::Role->initialize(scalar caller);
@@ -45,6 +61,7 @@ sub before {
     for (@_) {
         $meta->add_before_method_modifier($_ => $code);
     }
+    return;
 }
 
 sub after {
@@ -54,6 +71,7 @@ sub after {
     for (@_) {
         $meta->add_after_method_modifier($_ => $code);
     }
+    return;
 }
 
 sub around {
@@ -63,6 +81,7 @@ sub around {
     for (@_) {
         $meta->add_around_method_modifier($_ => $code);
     }
+    return;
 }
 
 
@@ -74,6 +93,7 @@ sub super {
 sub override {
     # my($name, $code) = @_;
     Mouse::Meta::Role->initialize(scalar caller)->add_override_method_modifier(@_);
+    return;
 }
 
 # We keep the same errors messages as Moose::Role emits, here.
@@ -85,26 +105,11 @@ sub augment {
     Carp::croak "Roles cannot support 'augment'";
 }
 
-sub has {
-    my $meta = Mouse::Meta::Role->initialize(scalar caller);
-    my $name = shift;
-
-    $meta->add_attribute($_ => @_) for ref($name) ? @{$name} : $name;
-}
-
-sub extends  {
-    Carp::croak "Roles do not support 'extends'"
-}
-
-sub with     {
-    my $meta = Mouse::Meta::Role->initialize(scalar caller);
-    Mouse::Util::apply_all_roles($meta->name, @_);
-}
-
 sub requires {
     my $meta = Mouse::Meta::Role->initialize(scalar caller);
     $meta->throw_error("Must specify at least one method") unless @_;
     $meta->add_required_methods(@_);
+    return;
 }
 
 sub excludes {
@@ -143,7 +148,7 @@ Mouse::Role - The Mouse Role
 
 =head1 VERSION
 
-This document describes Mouse version 0.40
+This document describes Mouse version 0.41
 
 =head1 SYNOPSIS
 
@@ -158,18 +163,15 @@ Returns this role's metaclass instance.
 
 =head2 C<< before (method|methods) -> CodeRef >>
 
-Sets up a B<before> method modifier. See L<Moose/before> or
-L<Class::Method::Modifiers/before>.
+Sets up a B<before> method modifier. See L<Moose/before>.
 
 =head2 C<< after (method|methods) => CodeRef >>
 
-Sets up an B<after> method modifier. See L<Moose/after> or
-L<Class::Method::Modifiers/after>.
+Sets up an B<after> method modifier. See L<Moose/after>.
 
 =head2 C<< around (method|methods) => CodeRef >>
 
-Sets up an B<around> method modifier. See L<Moose/around> or
-L<Class::Method::Modifiers/around>.
+Sets up an B<around> method modifier. See L<Moose/around>.
 
 =head2 C<super>
 
